package com.g1.one_target_mobile_sdk

import android.app.Activity
import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.g1.onetargetsdk.core.Analytics
import com.g1.onetargetsdk.core.Configuration
import com.g1.onetargetsdk.core.IAM
import com.g1.onetargetsdk.model.IAMData
import com.google.gson.Gson
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** OneTargetMobileSdkPlugin */
class OneTargetMobileSdkPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    companion object {
        const val ENV_DEV = 1
        const val ENV_STAG = 2
        const val ENV_PROD = 3
        const val ERROR_CODE_CONTEXT_INVALID = "69"
    }

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private var activity: Activity? = null
    private var context: Context? = null

    private fun log(msg: String) {
        Log.e("g1mobile", msg)
    }

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, Const.sdk)
        channel.setMethodCallHandler(this)
        this.context = flutterPluginBinding.applicationContext
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            Const.getPlatformVersion -> {
                result.success("Android ${android.os.Build.VERSION.RELEASE}")
            }
            Const.setupTracking -> {
                setupG1SDK(call, result)
            }
            Const.trackEvent -> {
                trackEvent(call, result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun setupG1SDK(call: MethodCall, result: Result) {
        if (context == null) {
            result.error(
                ERROR_CODE_CONTEXT_INVALID,
                "Invalid context",
                "context == null"
            )
            return
        }
        val configuration = Configuration(context!!)
        val arguments = call.arguments as Map<*, *>?
//        log("setupTracking $arguments")
        arguments?.let { map ->
            when (map["env"] as Int?) {
                ENV_DEV -> {
                    configuration.setEnvironmentDev()
                }
                ENV_STAG -> {
                    configuration.setEnvironmentStag()
                }
                ENV_PROD -> {
                    configuration.setEnvironmentProd()
                }
                else -> {
                    configuration.setEnvironmentProd()
                }
            }
            val writeKey = map["writeKey"] as String?
            configuration.writeKey = writeKey
//            log("setupTracking writeKey $writeKey")

            val oneTargetAppPushID = map["oneTargetAppPushID"] as String?
            configuration.oneTargetAppPushID = oneTargetAppPushID
            log("setupTracking oneTargetAppPushID $oneTargetAppPushID")

            val isShowLog = map["isShowLog"] as Boolean?
            configuration.isShowLog = isShowLog ?: false

            val isEnableIAM = map["isEnableIAM"] as Boolean?
            configuration.isEnableIAM = isEnableIAM ?: true

            var maxNumberApiPolling = map["maxNumberApiPolling"] as Int?
            if (maxNumberApiPolling == null) {
                maxNumberApiPolling = Configuration.DEFAULT_MAX_NUMBER_API_POLLING
            }
            configuration.maxNumberApiPolling = maxNumberApiPolling
        }

        configuration.onShowIAM = { htmlContent: String, iamData: IAMData ->
//            log(">>>flutter onShowIAM htmlContent $htmlContent")
//            log(">>>flutter onShowIAM iamData $iamData")
            IAM.showIAMDialog(activity!!, htmlContent, iamData)
        }
        configuration.onMsg = { msg: String ->
//            log(">>>flutter onMsg $msg")
            try {
                channel.invokeMethod(Const.logBack, msg)
            } catch (e: Exception) {
                Handler(Looper.getMainLooper()).post {
                    channel.invokeMethod(Const.logBack, msg)
                }
            }
        }
        val isSetupSuccess = Analytics.setup(configuration, context!!)
        if (isSetupSuccess) {
            result.success(true)
        } else {
            result.success(false)
        }
    }

    private fun trackEvent(call: MethodCall, result: Result) {
        val arguments = call.arguments as Map<*, *>?
//        log("trackEvent $arguments")
        if (arguments == null) {
//            log("null -> return")
            result.success(null)
            return
        }
        arguments.let { map ->
            val identityId = map["identityId"] as HashMap<String, Any>?
            val profile = map["profile"] as ArrayList<HashMap<String, Any>>?
            val eventName = map["eventName"] as String?
            val eventData = map["eventData"] as HashMap<String, Any>?

//            log("identityId $identityId")
//            log("profile profile")
//            log("eventName $eventName")
//            log("eventData $eventData")

            var mInput: Any? = null
            val gson = Gson()
            Analytics.trackEvent(
                identityId = identityId,
                profile = profile,
                eventName = eventName,
                eventData = eventData,
                onPreExecute = { input ->
                    mInput = input
//                    result.success(
//                        hashMapOf(
//                            "input" to Gson().toJson(input),
//                        )
//                    )
                },
                onResponse = { isSuccessful, code, response ->
                    result.success(
                        hashMapOf(
                            "input" to gson.toJson(mInput),
                            "isSuccessful" to isSuccessful,
                            "code" to code,
                            "response" to gson.toJson(response),
                        )
                    )
                },
                onFailure = { t ->
                    result.success(
                        hashMapOf(
                            "input" to gson.toJson(mInput),
                            "error" to t.message,
                        )
                    )
                }
            )
        }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        //do nothing
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        onAttachedToActivity(binding)
    }

    override fun onDetachedFromActivity() {
        //do nothing
    }
}
